;==============================================================================
    include \masm32\include\masm32rt.inc
    include \masm32\include\Iphlpapi.inc
    includelib \masm32\lib\Iphlpapi.lib
    include \masm32\include\ws2_32.inc
    includelib \masm32\lib\ws2_32.lib
;==============================================================================

printf MACRO format:REQ, args:VARARG
    IFNB <args>
        invoke crt_printf, cfm$(format), args
    ELSE
        invoke crt_printf, cfm$(format)
    ENDIF
ENDM

;==============================================================================

comment |
#define MIB_IPADDR_PRIMARY      0x0001  // Primary ipaddr
#define MIB_IPADDR_DYNAMIC      0x0004  // Dynamic ipaddr
#define MIB_IPADDR_DISCONNECTED 0x0008  // Address is on disconnected interface
#define MIB_IPADDR_DELETED      0x0040  // Address being deleted
#define MIB_IPADDR_TRANSIENT    0x0080  // Transient address
typedef struct _MIB_IPADDRROW
{
    DWORD        dwAddr;
    DWORD        dwIndex;
    DWORD        dwMask;
    DWORD        dwBCastAddr;
    DWORD        dwReasmSize;
    unsigned short    unused1;
    unsigned short    wType;
} MIB_IPADDRROW, *PMIB_IPADDRROW;
typedef struct _MIB_IPADDRTABLE
{
    DWORD         dwNumEntries;
    MIB_IPADDRROW table[ANY_SIZE];
} MIB_IPADDRTABLE, *PMIB_IPADDRTABLE;
|

MIB_IPADDR_PRIMARY      equ 1
MIB_IPADDR_DYNAMIC      equ 4
MIB_IPADDR_DISCONNECTED equ 8
MIB_IPADDR_DELETED      equ 40h
MIB_IPADDR_TRANSIENT    equ 80h

MIB_IPADDRROW STRUCT
    dwAddr      DWORD ?
    dwIndex     DWORD ?
    dwMask      DWORD ?
    dwBCastAddr DWORD ?
    dwReasmSize DWORD ?
    unused1     WORD  ?
    wType       WORD  ?
MIB_IPADDRROW ENDS

MIB_IPADDRTABLE STRUCT
    dwNumEntries DWORD ?
    table        MIB_IPADDRROW <>
MIB_IPADDRTABLE ENDS

;==============================================================================
    .data
        dwSize   dd 0
        lpMsgBuf dd 0
        IPAddr   in_addr <>
    .code
;==============================================================================
start:
;==============================================================================
    printf "SIZEOF MIB_IPADDRTABLE: \t%d\n", SIZEOF MIB_IPADDRTABLE
    printf "SIZEOF MIB_IPADDRROW:   \t%d\n", SIZEOF MIB_IPADDRROW
    mov ebx, alloc(SIZEOF MIB_IPADDRTABLE)
    invoke GetIpAddrTable, ebx, ADDR dwSize, 0
    .IF eax == ERROR_INSUFFICIENT_BUFFER
        printf "Required buffer size:    \t%d\n\n", dwSize
        free ebx
        mov ebx, alloc(dwSize)
    .ENDIF
    .IF ebx == 0
        inkey "allocation failed, press any key to exit..."
        exit 1
    .ENDIF
    invoke GetIpAddrTable, ebx, ADDR dwSize, 0
    .IF eax != NO_ERROR
        invoke FormatMessage,FORMAT_MESSAGE_ALLOCATE_BUFFER or \
                             FORMAT_MESSAGE_FROM_SYSTEM or \
                             FORMAT_MESSAGE_IGNORE_INSERTS,
                             0, eax, 0, ADDR lpMsgBuf, 0, 0
        printf "%s\n", lpMsgBuf
        invoke LocalFree, lpMsgBuf
        inkey "Press any key to exit..."
        exit 1
    .ENDIF
    printf "\tNum Entries: %d\n",[ebx].MIB_IPADDRTABLE.dwNumEntries
    xor esi, esi
    mov edi, ebx
    .WHILE esi < [ebx].MIB_IPADDRTABLE.dwNumEntries

        printf "\n\tInterface Index[%d]:\t%d\n", \
                esi,[edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.dwIndex

        push [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.dwAddr
        pop  IPAddr.S_un.S_addr

        ;------------------------------------------------------------------
        ; The inet_ntoa function expects a 4-byte in_addr structure passed
        ; directly on the stack. The MASM32 inet_ntoa prototype specifies
        ; a DWORD, and even though the sizes match, I had to add the
        ; DWORD PTR to the following invoke statements to avoid having ML
        ; (6.14) return an NVOKE argument type mismatch error.
        ;------------------------------------------------------------------

        invoke inet_ntoa, DWORD PTR IPAddr
        printf "\tIP Address[%d]:     \t%s\n", esi, eax

        push [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.dwMask
        pop  IPAddr.S_un.S_addr
        invoke inet_ntoa, DWORD PTR IPAddr
        printf "\tSubnet Mask[%d]:    \t%s\n", esi, eax

        push [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.dwBCastAddr
        pop  IPAddr.S_un.S_addr
        invoke inet_ntoa, DWORD PTR IPAddr
        mov edx, [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.dwBCastAddr
        printf "\tBroadCast[%d]:      \t%s (%ld%)\n", esi, eax, edx

        mov eax, [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.dwReasmSize
        printf "\tReassembly size[%d]:\t%ld\n", esi, eax
        printf "\tType and State[%d]:", esi

        .IF [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.wType & MIB_IPADDR_PRIMARY
            printf "\tPrimary IP Address"
        .ENDIF
        .IF [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.wType & MIB_IPADDR_DYNAMIC
            printf "\tDynamic IP Address"
        .ENDIF
        .IF [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.wType & MIB_IPADDR_DISCONNECTED
            printf "\tAddress is on disconnected interface",0
        .ENDIF
        .IF [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.wType & MIB_IPADDR_DELETED
            printf "\tAddress is being deleted"
        .ENDIF
        .IF [edi].MIB_IPADDRTABLE.table.MIB_IPADDRROW.wType & MIB_IPADDR_TRANSIENT
            printf "\tTransient address"
        .ENDIF

        printf "\n\n"
        inc esi
        add edi, SIZEOF MIB_IPADDRROW
    .ENDW

    free ebx
    inkey "Press any key to exit..."
    exit 0
;==============================================================================
end start
